pub enum ParseNonMaxError {
Parse(ParseIntError),
IsMax,
}Expand description
Error type returned when parsing a NonMax value from a string fails.
Variants
Parse(ParseIntError)
The value could not be parsed as the underlying integer type.
IsMax
The parsed value was the maximum value for the type.