NonMax

Struct NonMax 

pub struct NonMax<T: NonMaxItem>(/* private fields */);
A wrapper type for an integer that cannot be its maximum value.

This type leverages Rust’s NonZero optimization by mapping the maximum value to zero internally. As a result, Option<NonMax<T>> has the same size as the underlying primitive type T.

Examples

assert_eq!(size_of::<NonMaxU32>(), 4);
assert_eq!(size_of::<Option<NonMaxU32>>(), 4);

Implementations

Source

impl<T: NonMaxItem + Copy> NonMax<T>

pub fn new(value: T) -> Option<Self>

Creates a new NonMax if the given value is not the maximum value.

Examples
assert!(NonMaxU8::new(254).is_some());
assert!(NonMaxU8::new(255).is_none());
pub fn get(&self) -> T

Returns the underlying primitive value.

Examples
let x = NonMaxU32::new(123).unwrap();
assert_eq!(x.get(), 123);
Source

impl<T: NonMaxItem + Copy + PartialEq> NonMax<T>

pub fn is_min(&self) -> bool

Returns true if this is the minimum value.

pub fn is_max(&self) -> bool

Returns true if this is the maximum possible value for this type.

pub fn is_zero(&self) -> bool

Returns true if the value is zero.

Source

impl<T: NonMaxItem + Copy> NonMax<T>

pub fn checked_add(self, rhs: Self) -> Option<Self>

Checked integer addition. Computes self + rhs, returning None if overflow occurred or if the result is the maximum value.

pub fn checked_sub(self, rhs: Self) -> Option<Self>

Checked integer subtraction. Computes self - rhs, returning None if overflow occurred or if the result is the maximum value.

pub fn checked_mul(self, rhs: Self) -> Option<Self>

Checked integer multiplication. Computes self * rhs, returning None if overflow occurred or if the result is the maximum value.

pub fn checked_div(self, rhs: Self) -> Option<Self>

Checked integer division. Computes self / rhs, returning None if the divisor is zero or if the result is the maximum value.

pub fn checked_rem(self, rhs: Self) -> Option<Self>

Checked integer remainder. Computes self % rhs, returning None if the divisor is zero or if the result is the maximum value.

pub fn checked_add_val(self, rhs: T) -> Option<Self>

Checked addition with a primitive value.

Examples
let x = NonMaxU8::new(100).unwrap();
assert_eq!(x.checked_add_val(50).unwrap().get(), 150);
assert!(x.checked_add_val(155).is_none()); // 255 is MAX
pub fn checked_sub_val(self, rhs: T) -> Option<Self>

Checked subtraction with a primitive value.

pub fn checked_mul_val(self, rhs: T) -> Option<Self>

Checked multiplication with a primitive value.

pub fn checked_div_val(self, rhs: T) -> Option<Self>

Checked division with a primitive value.

pub fn checked_rem_val(self, rhs: T) -> Option<Self>

Checked remainder with a primitive value.

Source

impl NonMax<u8>

pub const MIN: Self

The minimum value for this type.

pub const MAX: Self

The maximum value for this type.

pub const ZERO: Self

The zero value for this type.

pub const unsafe fn new_unchecked(value: u8) -> Self

Creates a new NonMax without checking the value.

Safety

The value must not be the maximum value of the underlying type.

Source

impl NonMax<u16>

pub const MIN: Self

The minimum value for this type.

pub const MAX: Self

The maximum value for this type.

pub const ZERO: Self

The zero value for this type.

pub const unsafe fn new_unchecked(value: u16) -> Self

Creates a new NonMax without checking the value.

Safety

The value must not be the maximum value of the underlying type.

Source

impl NonMax<u32>

pub const MIN: Self

The minimum value for this type.

pub const MAX: Self

The maximum value for this type.

pub const ZERO: Self

The zero value for this type.

pub const unsafe fn new_unchecked(value: u32) -> Self

Creates a new NonMax without checking the value.

Safety

The value must not be the maximum value of the underlying type.

Source

impl NonMax<u64>

pub const MIN: Self

The minimum value for this type.

pub const MAX: Self

The maximum value for this type.

pub const ZERO: Self

The zero value for this type.

pub const unsafe fn new_unchecked(value: u64) -> Self

Creates a new NonMax without checking the value.

Safety

The value must not be the maximum value of the underlying type.

Source

impl NonMax<u128>

pub const MIN: Self

The minimum value for this type.

pub const MAX: Self

The maximum value for this type.

pub const ZERO: Self

The zero value for this type.

pub const unsafe fn new_unchecked(value: u128) -> Self

Creates a new NonMax without checking the value.

Safety

The value must not be the maximum value of the underlying type.

Source

impl NonMax<usize>

pub const MIN: Self

The minimum value for this type.

pub const MAX: Self

The maximum value for this type.

pub const ZERO: Self

The zero value for this type.

pub const unsafe fn new_unchecked(value: usize) -> Self

Creates a new NonMax without checking the value.

Safety

The value must not be the maximum value of the underlying type.

Source

impl NonMax<i8>

pub const MIN: Self

The minimum value for this type.

pub const MAX: Self

The maximum value for this type.

pub const ZERO: Self

The zero value for this type.

pub const unsafe fn new_unchecked(value: i8) -> Self

Creates a new NonMax without checking the value.

Safety

The value must not be the maximum value of the underlying type.

Source

impl NonMax<i16>

pub const MIN: Self

The minimum value for this type.

pub const MAX: Self

The maximum value for this type.

pub const ZERO: Self

The zero value for this type.

pub const unsafe fn new_unchecked(value: i16) -> Self

Creates a new NonMax without checking the value.

Safety

The value must not be the maximum value of the underlying type.

Source

impl NonMax<i32>

pub const MIN: Self

The minimum value for this type.

pub const MAX: Self

The maximum value for this type.

pub const ZERO: Self

The zero value for this type.

pub const unsafe fn new_unchecked(value: i32) -> Self

Creates a new NonMax without checking the value.

Safety

The value must not be the maximum value of the underlying type.

Source

impl NonMax<i64>

pub const MIN: Self

The minimum value for this type.

pub const MAX: Self

The maximum value for this type.

pub const ZERO: Self

The zero value for this type.

pub const unsafe fn new_unchecked(value: i64) -> Self

Creates a new NonMax without checking the value.

Safety

The value must not be the maximum value of the underlying type.

Source

impl NonMax<i128>

pub const MIN: Self

The minimum value for this type.

pub const MAX: Self

The maximum value for this type.

pub const ZERO: Self

The zero value for this type.

pub const unsafe fn new_unchecked(value: i128) -> Self

Creates a new NonMax without checking the value.

Safety

The value must not be the maximum value of the underlying type.

Source

impl NonMax<isize>

pub const MIN: Self

The minimum value for this type.

pub const MAX: Self

The maximum value for this type.

pub const ZERO: Self

The zero value for this type.

pub const unsafe fn new_unchecked(value: isize) -> Self

Creates a new NonMax without checking the value.

Safety

The value must not be the maximum value of the underlying type.